#import <Foundation/Foundation.h>
#import <CommonCrypto/CommonCrypto.h>

#import "Sync2chRequestSample.h"


//
// 以下のように開始します
// Sync2chRequestSample* sync2chClient = [[Sync2chRequestSample alloc] init];
// [sync2chClient startSync];
//



// スレ情報を格納するクラス
@interface ThreadInfo : NSObject<NSCoding> {
    
}
@property NSString* url;
@property NSString* title;
@property NSInteger readPosition;
@property NSInteger currentPosition;
@property NSInteger resCount;
@property NSMutableArray* myResList;

- (id) initWithUrl:(NSString*) url;
@end

@implementation ThreadInfo 
- (id) initWithUrl:(NSString*) url {
    if (self = [super init]) {
        _url = url;
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)decoder
{
    if (self = [super init]) {
        NSNumber *readPosNumber = [decoder decodeObjectForKey:@"readPos"];
        if (readPosNumber) {
            _readPosition  = [readPosNumber integerValue];
        }
       
        
        NSNumber *currentPosNumber = [decoder decodeObjectForKey:@"currentPos"];
        if (currentPosNumber) {
            _currentPosition  = [currentPosNumber integerValue];
        }
        
        NSNumber *resCountNumber = [decoder decodeObjectForKey:@"resCount"];
        if (resCountNumber) {
            _resCount  = [resCountNumber integerValue];
        }

        
         _title = [decoder decodeObjectForKey:@"title"];
         _url = [decoder decodeObjectForKey:@"url"];
    }
    return self;
}

- (void)encodeWithCoder:(NSCoder *)encoder {
    [encoder encodeObject:self.title forKey:@"title"];
    [encoder encodeObject:self.url forKey:@"url"];
    [encoder encodeObject:[NSNumber numberWithInteger:self.readPosition] forKey:@"readPos"];
    [encoder encodeObject:[NSNumber numberWithInteger:self.currentPosition] forKey:@"currentPos"];
    [encoder encodeObject:[NSNumber numberWithInteger:self.resCount] forKey:@"resCount"];
}

@end

@interface Sync2chRequestSample () {

}
// thのidに対する属性を保持しておく。
@property NSDictionary* idEntities;

// 現在ローカルで持っているスレッドのurlとThreadInfoオブジェクトの対応
@property NSDictionary* urlToThreads;

// 受け取ったThreadInfoを追加していく配列
@property NSMutableArray* receivedThreads;

@property NSString* currentCategory;

@end


@implementation Sync2chRequestSample

- (void) startSync {
    
    NSString *userId = @"<ユーザーIDを入力>";
    NSString *syncPass = @"<API接続用パスワードを入力>";
    
    NSString* idAndPass = [self encBase64:[NSString stringWithFormat:@"%@:%@", userId, syncPass]];

    NSString *syncNumber = [[NSUserDefaults standardUserDefaults] stringForKey:@"sync_number"];
    NSString *clientId = [[NSUserDefaults standardUserDefaults] stringForKey:@"client_id"];
    if (syncNumber == nil) syncNumber = @"0";
    if (clientId == nil) clientId = @"0";

    // 構造を持たないので、 配列で保持する
    NSArray* threads  = [self loadThreads];

    NSString* xml = [self createRequestXML:syncNumber clientId:clientId withThreads:threads];

    NSString* urlstr = @"https://sync2ch.com/api/sync3";
    
    NSURL *nsurl = [NSURL URLWithString:urlstr];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:nsurl];
    request.HTTPMethod = @"POST";

    [request addValue:[NSString stringWithFormat:@"Basic %@", idAndPass] forHTTPHeaderField:@"Authorization"];
    
    request.HTTPBody = [xml dataUsingEncoding:NSUTF8StringEncoding];
    
    NSURLResponse *response = nil;
    NSError *error = nil;
    NSData *data = [ NSURLConnection sendSynchronousRequest : request
                                          returningResponse : &response
                                                      error : &error
                    ];
    
    NSHTTPURLResponse* hres = (NSHTTPURLResponse*) response;
    NSLog(@"sync response statusCode = %ld", (long)hres.statusCode);
    
    // error
    NSString *errorStr = [error localizedDescription];
    if (0 < [errorStr length]) {
        return;
    }
    
    [self parseResponseXML:data withThreads:threads];
}

- (NSString*) pathForThreads {
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *path = [paths objectAtIndex:0];

    return [path stringByAppendingPathComponent: @"threads"];
}

- (NSArray*) loadThreads {
    
    NSString* savePath =    [self pathForThreads];
    
    NSFileManager*fm = [NSFileManager defaultManager];
    if ([fm fileExistsAtPath: savePath] == NO) {
        return nil;
    }
    
    NSArray* threads = [NSKeyedUnarchiver unarchiveObjectWithFile:savePath];
    if (threads) {
        for (ThreadInfo* thInfo in threads) {
            NSLog(@"th title=%@ url=%@", thInfo.title, thInfo.url);
        }
    }
    
    return threads;
}

- (void) saveThreads:(NSArray*)threads {
    BOOL successful = [NSKeyedArchiver archiveRootObject:threads toFile:[self pathForThreads]];
}

- (NSString*) createRequestXML:(NSString*)syncNumber clientId:(NSString*)clientId withThreads:(NSArray*)threads {

    NSMutableString * xml = [NSMutableString string];
    
    [xml appendFormat:@"<?xml version=\"1.0\" encoding=\"utf-8\"?>" ];
    [xml appendFormat:@"<sync2ch_request client_id=\"%@\" sync_number=\"%@\" client_name=\"CocoaSync2chSample\" client_version=\"1.0.0\">\n" , clientId, syncNumber];


    [xml appendFormat:@"<thread_group category=\"favorite\" struct=\"CocoaSync2chSample\">\n" ];

    for (ThreadInfo* th in threads) {
        [xml appendFormat:@"<th url=\"%@\" title=\"%@\" read=\"%lu\" now=\"%lu\" count=\"%lu\"/>\n", th.url, [self xmlSimpleEscape:th.title], (long)th.readPosition, (long)th.currentPosition, th.resCount ];

    }

    [xml appendFormat:@"</thread_group>\n" ];
    [xml appendString:@"</sync2ch_request>"];

    return xml;
}


- (NSString*) encBase64:(NSString*) text {
    NSData* data  = [text dataUsingEncoding:NSUTF8StringEncoding];
    NSString *encoded_data;
    if ([data respondsToSelector:@selector(base64EncodedStringWithOptions:)]) {
        encoded_data = [data base64EncodedStringWithOptions:kNilOptions]; // iOS7 and later
    } else {
        encoded_data = [data base64Encoding]; // iOS6 and prior
    }

    return encoded_data;
}

- (void) parseResponseXML:(NSData*) data  withThreads:(NSArray*)threads {
    NSXMLParser* parser = [[NSXMLParser alloc] initWithData:data];
    parser.delegate = self;

    self.currentCategory = nil;
    self.urlToThreads = [NSMutableDictionary dictionary];
    for (ThreadInfo * thInfo in threads) {
        if (thInfo.url )
        [self.urlToThreads setValue:thInfo forKey:thInfo.url];
    }
  
    [parser parse];

    if (self.receivedThreads) {
        for (ThreadInfo* thInfo in self.receivedThreads) {
            NSLog(@"th title=%@ url=%@", thInfo.title, thInfo.url);
        }
        [self saveThreads:self.receivedThreads];
    }
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName
  namespaceURI:(NSString *)namespaceURI
 qualifiedName:(NSString *)qName
    attributes:(NSDictionary *)attributeDict {
    
    NSLog(@"element = %@\n" ,elementName);
    
    if ([elementName isEqualToString:@"sync2ch_response"]) {
        
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        [defaults setObject:[attributeDict objectForKey:@"sync_number"] forKey:@"sync_number"];
        [defaults setObject:[attributeDict objectForKey:@"client_id"] forKey:@"client_id"];

    } else if ([elementName isEqualToString:@"entities"]) {
        self.currentCategory = @"entities";
        self.idEntities = [NSMutableDictionary dictionary];
        
    } else if ([elementName isEqualToString:@"thread_group"]) {
        self.currentCategory = [attributeDict objectForKey:@"category"];
        if ([self.currentCategory isEqualToString:@"favorite"]) {
            self.receivedThreads = [NSMutableArray array];
        }
        
    } else if ([elementName isEqualToString:@"th"]) {
        if ([self.currentCategory isEqualToString:@"entities"]) {
            //in entities
            [self.idEntities setValue:attributeDict forKey:[attributeDict objectForKey:@"id"]];

        } else if ([self.currentCategory isEqualToString:@"favorite"]) {
            NSDictionary * thAttributeDict = [self.idEntities objectForKey:[attributeDict objectForKey:@"id"]];
            NSString* url = [thAttributeDict  objectForKey:@"url"];
            NSString* title = [thAttributeDict  objectForKey:@"title"];
            NSString* read = [thAttributeDict  objectForKey:@"read"];
            NSString* now = [thAttributeDict  objectForKey:@"now"];
            NSString* count = [thAttributeDict  objectForKey:@"count"];
            NSString* status = [thAttributeDict  objectForKey:@"s"];
            
            NSLog(@"url = %@", url);

            ThreadInfo* ownThread = [self.urlToThreads  objectForKey:url];
            if (!ownThread) {
                ownThread = [[ThreadInfo alloc] initWithUrl:url];
            }

            if ([status isEqualToString:@"n"] == NO) {
                ownThread.title = title;
                ownThread.readPosition = [read integerValue];
                ownThread.currentPosition = [now integerValue];
                ownThread.resCount = [count integerValue];
            }

            [self.receivedThreads addObject:ownThread];

        }
        
    }
}


- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName
  namespaceURI:(NSString *)namespaceURI
 qualifiedName:(NSString *)qName {
    
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
    
}



- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
    
}

- (NSString*) xmlSimpleEscape:(NSString*)str{

    if (str == nil) return @"";
    
    
    NSMutableString* result = [NSMutableString stringWithString:str];
    [result replaceOccurrencesOfString:@"&"  withString:@"&amp;"  options:NSLiteralSearch range:NSMakeRange(0, [str length])];
    [result replaceOccurrencesOfString:@"\"" withString:@"&quot;" options:NSLiteralSearch range:NSMakeRange(0, [str length])];
    [result replaceOccurrencesOfString:@"'"  withString:@"&#x27;" options:NSLiteralSearch range:NSMakeRange(0, [str length])];
    
    [result replaceOccurrencesOfString:@">"  withString:@"&gt;"   options:NSLiteralSearch range:NSMakeRange(0, [str length])];
    [result replaceOccurrencesOfString:@"<"  withString:@"&lt;"   options:NSLiteralSearch range:NSMakeRange(0, [str length])];
    
    
    
    return result;
    
}


@end
